@@ -206,6 +206,9 @@ struct Display
 	struct Mux	*mux;
 	int		srvfd;
 	int		dpi;
+	
+	Font	*firstfont;
+	Font	*lastfont;
 };
 
 struct Image
@@ -305,6 +308,7 @@ struct Cachesubf
 struct Font
 {
 	char		*name;
+	char		*namespec;
 	Display		*display;
 	short		height;	/* max height of image, interline spacing */
 	short		ascent;	/* top of image to baseline */
@@ -314,10 +318,20 @@ struct Font
 	int		maxdepth;	/* maximum depth of all loaded subfonts */
 	int		ncache;	/* size of cache */
 	int		nsubf;	/* size of subfont list */
+	int		scale;	/* pixel scaling to apply */
 	Cacheinfo	*cache;
 	Cachesubf	*subf;
 	Cachefont	**sub;	/* as read from file */
 	Image		*cacheimage;
+	
+	/* doubly linked list of fonts known to display */
+	int ondisplaylist;
+	Font *next;
+	Font *prev;
+	
+	/* on hi-dpi systems, one of these is set to f and the other is the other-dpi version of f */
+	Font	*lodpi;
+	Font	*hidpi;
 };
 
 #define	Dx(r)	((r).max.x-(r).min.x)
@@ -459,6 +473,7 @@ extern void	borderop(Image*, Rectangle, int, Image*, Point, Drawop);
  * Font management
  */
 extern Font*	openfont(Display*, char*);
+extern int	parsefontscale(char*, char**);
 extern Font*	buildfont(Display*, char*, char*);
 extern void	freefont(Font*);
 extern Font*	mkfont(Subfont*, Rune);
@@ -487,6 +502,8 @@ extern Subfont*	getdefont(Display*);
 extern void		lockdisplay(Display*);
 extern void	unlockdisplay(Display*);
 extern int		drawlsetrefresh(u32int, int, void*, void*);
+extern void	loadhidpi(Font*);
+extern void	swapfont(Font*, Font**, Font**);
 
 /*
  * Predefined 
@@ -508,6 +525,8 @@ extern	int	_drawdebug;	/* set to 1 to see errors from flushimage */
 extern	void	_setdrawop(Display*, Drawop);
 extern	Display	*_initdisplay(void(*)(Display*,char*), char*);
 
+extern	void	needdisplay(void); /* call instead of initdraw to get (null) variable linked in */
+
 #define	BGSHORT(p)		(((p)[0]<<0) | ((p)[1]<<8))
 #define	BGLONG(p)		((BGSHORT(p)<<0) | (BGSHORT(p+2)<<16))
 #define	BPSHORT(p, v)		((p)[0]=(v), (p)[1]=((v)>>8))
